@@ -12,20 +12,23 @@ has customFunctions => ( is => 'rw', isa => 'Maybe[Str]' );
 
 has useSafeJail => ( is => 'rw', isa => 'Maybe[Int]' );
 
-has safe => ( is => 'rw' );
+has jail => ( is => 'rw' );
 
-our $VERSION = '1.3.1';
+our $VERSION = '1.4.0';
 
-# for accessing $datas and $apacheRequest
-use Lemonldap::NG::Handler::Main ':jailSharedVars';
+use Lemonldap::NG::Handler::Main '$datas';
+use Lemonldap::NG::Handler::API ':functions';
 
-## @imethod protected build_safe()
+## @imethod protected build_jail()
 # Build and return the security jail used to compile rules and headers.
 # @return Safe object
-sub build_safe {
+sub build_jail {
     my $self = shift;
 
-    return $self->safe if ( $self->safe );
+    return $self->jail
+      if ( $self->jail
+        && $self->jail->useSafeJail == $self->useSafeJail
+        && $self->jail->customFunctions == $self->customFunctions );
 
     $self->useSafeJail(1) unless defined $self->useSafeJail;
 
@@ -43,58 +46,64 @@ sub build_safe {
         }
         next if ( $self->can($_) );
         eval "sub $_ {
-            my \$uri = \$Lemonldap::NG::Handler::Main::apacheRequest->unparsed_uri();
-            Apache2::URI::unescape_url(\$uri);
-            return $sub(\$uri, \@_)
-            }";
+            my \$uri = Lemonldap::NG::Handler::API::${Lemonldap::NG::Handler::API::mode}::uri_with_args();
+            return $sub(\$uri,\@_)
+        }";
         Lemonldap::NG::Handler::Main::Logger->lmLog( $@, 'error' ) if ($@);
+        $_ = "&$_";
     }
 
     if ( $self->useSafeJail ) {
-        $self->safe( Safe->new );
-        $self->safe->share_from( 'main', ['%ENV'] );
+        $self->jail( Safe->new );
+        $self->jail->share_from( 'main', ['%ENV'] );
     }
     else {
-        $self->safe($self);
+        $self->jail($self);
     }
 
     # Share objects with Safe jail
-    $self->safe->share_from( 'Lemonldap::NG::Common::Safelib',
+    $self->jail->share_from( 'Lemonldap::NG::Common::Safelib',
         $Lemonldap::NG::Common::Safelib::functions );
 
-    $self->safe->share_from( 'Lemonldap::NG::Handler::Main',
-        [ '$datas', '$apacheRequest', '&ip', '&portal' ] );
-    $self->safe->share(@t);
-    $self->safe->share_from( 'MIME::Base64', ['&encode_base64'] );
+    $self->jail->share_from( 'Lemonldap::NG::Handler::Main', ['$datas'] );
+    $self->jail->share_from(
+        'Lemonldap::NG::Handler::API',
+        [
+            qw( &hostname &remote_ip &uri &uri_with_args
+              &unparsed_uri &args &method &header_in   )
+        ]
+    );
+    $self->jail->share_from(__PACKAGE__,[@t]);
+    $self->jail->share_from( 'MIME::Base64', ['&encode_base64'] );
 
-    return $self->safe;
+    return $self->jail;
 }
 
 ## @method reval
 # Fake reval method if useSafeJail is off
 sub reval {
-    my ( $self, $e ) = splice @_;
+    my ( $self, $e ) = @_;
     return eval $e;
 }
 
 ## @method wrap_code_ref
 # Fake wrap_code_ref method if useSafeJail is off
 sub wrap_code_ref {
-    my ( $self, $e ) = splice @_;
+    my ( $self, $e ) = @_;
     return $e;
 }
 
 ## @method share
 # Fake share method if useSafeJail is off
 sub share {
-    my ( $self, @vars ) = splice @_;
+    my ( $self, @vars ) = @_;
     $self->share_from( scalar(caller), \@vars );
 }
 
 ## @method share_from
 # Fake share_from method if useSafeJail is off
 sub share_from {
-    my ( $self, $pkg, $vars ) = splice @_;
+    my ( $self, $pkg, $vars ) = @_;
 
     no strict 'refs';
     foreach my $arg (@$vars) {
@@ -117,7 +126,7 @@ sub share_from {
 # Build and return restricted eval command with SAFEWRAP, if activated
 # @return evaluation of $reval or $reval2
 sub jail_reval {
-    my ( $self, $reval ) = splice @_;
+    my ( $self, $reval ) = @_;
 
     # if nothing is returned by reval, add the return statement to
     # the "no safe wrap" reval
@@ -128,8 +137,8 @@ sub jail_reval {
 
     return (
         SAFEWRAP
-        ? $self->safe->wrap_code_ref( $self->safe->reval($reval) )
-        : $self->safe->reval($nosw_reval)
+        ? $self->jail->wrap_code_ref( $self->jail->reval($reval) )
+        : $self->jail->reval($nosw_reval)
     );
 
 }
