@@ -7,9 +7,7 @@ use strict;
 use POSIX qw(setuid setgid);
 use Data::Dumper;
 
-#inherits Cache::Cache
-
-our $VERSION = '1.4.0';
+our $VERSION = '1.4.1';
 
 our $status   = {};
 our $activity = [];
@@ -106,19 +104,14 @@ eval {
     setuid( ( getpwnam( $ENV{APACHE_RUN_USER} ) )[2] );
 };
 
-## @rfn void run(string localStorage, hashRef localStorageOptions)
+## @rfn void run()
 # Main.
 # Reads requests from STDIN to :
 # - update counts
 # - display results
 sub run {
-    my ( $localStorage, $localStorageOptions ) = ( shift, shift );
-    my $refLocalStorage;
-    eval
-"use $localStorage; \$refLocalStorage = new $localStorage(\$localStorageOptions);";
-    die($@) if ($@);
     $| = 1;
-    my ( $lastMn, $mn, $count );
+    my ( $lastMn, $mn, $count, $cache );
     while (<STDIN>) {
         $mn = int( time / 60 ) - $start + 1;
 
@@ -158,6 +151,17 @@ sub run {
             $activity->[0]->{$code}++;
         }
 
+        elsif (/^RELOADCACHE(?:\s+(\S+?),(\S+))?$/) {
+            if ( my ( $cacheModule, $cacheOptions ) = ( $1, $2 ) ) {
+                eval "use $cacheModule;"
+                  . "\$cache = new $cacheModule(\$cacheOptions);";
+                print STDERR "$@\n" if ($@);    # TODO: use lmLog instead
+            }
+            else {
+                $cache = undef;
+            }
+        }
+
         # Status requests
 
         # $args contains parameters passed to url status page (a=1 for example
@@ -170,7 +174,8 @@ sub run {
             &head;
 
             my ( $c, $m, $u );
-            while ( my ( $user, $v ) = each( %{ $status->{user} } ) ) {
+            foreach my $user ( keys %{ $status->{user} } ) {
+                my $v = $status->{user}->{$user};
                 $u++ unless ( $user =~ /^\d+\.\d+\.\d+\.\d+$/ );
 
                 # Total requests
@@ -214,11 +219,11 @@ sub run {
                 print "<div id=\"users\"><p>\nTotal users : $u\n</p></div>\n";
 
                 # Local cache
-                my @t =
-                  $refLocalStorage->get_keys(
-                    $localStorageOptions->{namespace} );
-                print "<div id=\"cache\"><p>\nLocal Cache : " . @t
-                  . " objects\n</p></div>\n";
+                if ($cache) {
+                    my @t = $cache->get_keys( $_[1]->{namespace} );
+                    print "<div id=\"cache\"><p>\nLocal Cache : " . @t
+                      . " objects\n</p></div>\n";
+                }
 
                 # Uptime
                 print "<div id=\"up\"><p>\nServer up for : "
@@ -296,7 +301,7 @@ sub timeUp {
 # @param $cat Category to display
 # @param $max Number of lines to display
 sub topByCat {
-    my ( $cat, $max ) = splice @_;
+    my ( $cat, $max ) = @_;
     my $i = 0;
     print "<pre>\n";
     foreach (
@@ -507,13 +512,13 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2008, 2009, 2010, 2012 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2008-2012 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
 
 =item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2010, 2011, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2010-2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
